@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 Red Hat, Inc.
+ * Copyright (C) 2003,2004 Red Hat, Inc.
  *
  * This is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Library General Public License as published by
@@ -16,7 +16,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vtefc.c,v 1.13 2003/05/28 23:45:09 nalin Exp $"
 
 #include "../config.h"
 
@@ -46,7 +45,7 @@ _vte_fc_weight_from_pango_weight(int weight)
 }
 
 static int
-_vte_fc_slant_from_pango_style(int style)
+_vte_fc_slant_from_pango_style(PangoStyle style)
 {
 	switch (style) {
 	case PANGO_STYLE_NORMAL:
@@ -62,20 +61,55 @@ _vte_fc_slant_from_pango_style(int style)
 	return FC_SLANT_ROMAN;
 }
 
+static int
+_vte_fc_width_from_pango_stretch(PangoStretch stretch)
+{
+	switch (stretch) {
+	case PANGO_STRETCH_ULTRA_CONDENSED:
+		return 60;
+		break;
+	case PANGO_STRETCH_EXTRA_CONDENSED:
+		return 70;
+		break;
+	case PANGO_STRETCH_CONDENSED:
+		return 80;
+		break;
+	case PANGO_STRETCH_SEMI_CONDENSED:
+		return 90;
+		break;
+	case PANGO_STRETCH_NORMAL:
+		return 100;
+		break;
+	case PANGO_STRETCH_SEMI_EXPANDED:
+		return 105;
+		break;
+	case PANGO_STRETCH_EXPANDED:
+		return 120;
+		break;
+	case PANGO_STRETCH_EXTRA_EXPANDED:
+		return 150;
+		break;
+	case PANGO_STRETCH_ULTRA_EXPANDED:
+		return 200;
+		break;
+	}
+	return 100;
+}
+
 static void
 _vte_fc_transcribe_from_pango_font_description(GtkWidget *widget,
 					       FcPattern *pattern,
 				       const PangoFontDescription *font_desc)
 {
-#if GTK_CHECK_VERSION(2,2,0)
 	GdkScreen *screen;
-#endif
 	const char *family = "monospace";
 	PangoLanguage *language;
 	double size = 10.0;
 	int pango_mask;
 	PangoContext *context;
-	int weight, style;
+	PangoWeight weight;
+	PangoStyle style;
+	PangoStretch stretch;
 
 	if (font_desc == NULL) {
 		return;
@@ -97,16 +131,12 @@ _vte_fc_transcribe_from_pango_font_description(GtkWidget *widget,
 	FcPatternAddDouble(pattern, FC_SIZE, size);
 
 	/* Set the language for the pattern. */
-#if GTK_CHECK_VERSION(2,2,0)
 	if (gtk_widget_has_screen(widget)) {
 		screen = gtk_widget_get_screen(widget);
 	} else {
 		screen = gdk_display_get_default_screen(gtk_widget_get_display(widget));
 	}
 	context = gdk_pango_context_get_for_screen(screen);
-#else
-	context = gdk_pango_context_get();
-#endif
 	language = pango_context_get_language(context);
 	if (pango_language_to_string(language) != NULL) {
 		FcPatternAddString(pattern, FC_LANG,
@@ -121,6 +151,12 @@ _vte_fc_transcribe_from_pango_font_description(GtkWidget *widget,
 				    _vte_fc_weight_from_pango_weight(weight));
 	}
 
+	if (pango_mask & PANGO_FONT_MASK_STRETCH) {
+		stretch = pango_font_description_get_stretch(font_desc);
+		FcPatternAddInteger(pattern, FC_WIDTH,
+				    _vte_fc_width_from_pango_stretch(stretch));
+	}
+
 	if (pango_mask & PANGO_FONT_MASK_STYLE) {
 		style = pango_font_description_get_style(font_desc);
 		FcPatternAddInteger(pattern, FC_SLANT,
@@ -131,27 +167,37 @@ _vte_fc_transcribe_from_pango_font_description(GtkWidget *widget,
 }
 
 static void
-_vte_fc_defaults_from_gtk(GtkWidget *widget, FcPattern *pattern)
+_vte_fc_set_antialias(FcPattern *pattern, VteTerminalAntiAlias antialias)
+{
+	FcBool aa;
+	if (antialias != VTE_ANTI_ALIAS_USE_DEFAULT) {
+		if (FcPatternGetBool(pattern, FC_ANTIALIAS, 0,
+				     &aa) != FcResultNoMatch) {
+			FcPatternDel(pattern, FC_ANTIALIAS);
+		}
+		aa = (antialias == VTE_ANTI_ALIAS_FORCE_ENABLE) ?
+		     FcTrue : FcFalse;
+		FcPatternAddBool(pattern, FC_ANTIALIAS, aa);
+	}
+}
+
+static void
+_vte_fc_defaults_from_gtk(GtkWidget *widget, FcPattern *pattern,
+			  VteTerminalAntiAlias explicit_antialias)
 {
 	GtkSettings *settings;
-#if GTK_CHECK_VERSION(2,2,0)
 	GdkScreen *screen;
-#endif
 	GObjectClass *klass;
 	int i, antialias = -1, hinting = -1, dpi = -1;
 	char *rgba = NULL, *hintstyle = NULL;
 
 	/* Add any defaults configured for GTK+. */
-#if GTK_CHECK_VERSION(2,2,0)
 	if (gtk_widget_has_screen(widget)) {
 		screen = gtk_widget_get_screen(widget);
 	} else {
 		screen = gdk_display_get_default_screen(gtk_widget_get_display(widget));
 	}
 	settings = gtk_settings_get_for_screen(screen);
-#else
-	settings = gtk_settings_get_default();
-#endif
 	if (settings == NULL) {
 		return;
 	}
@@ -176,6 +222,7 @@ _vte_fc_defaults_from_gtk(GtkWidget *widget, FcPattern *pattern)
 		FcPatternDel(pattern, FC_ANTIALIAS);
 		FcPatternAddBool(pattern, FC_ANTIALIAS, antialias > 0);
 	}
+	_vte_fc_set_antialias(pattern, explicit_antialias);
 
 	/* Pick up the configured DPI setting. */
 	if (dpi >= 0) {
@@ -259,7 +306,8 @@ _vte_fc_defaults_from_gtk(GtkWidget *widget, FcPattern *pattern)
 }
 
 static void
-_vte_fc_defaults_from_rdb(GtkWidget *widget, FcPattern *pattern)
+_vte_fc_defaults_from_rdb(GtkWidget *widget, FcPattern *pattern,
+			  VteTerminalAntiAlias explicit_antialias)
 {
 	FcBool fcb;
 	double fcd;
@@ -277,6 +325,16 @@ _vte_fc_defaults_from_rdb(GtkWidget *widget, FcPattern *pattern)
 		antialias = _vte_rdb_get_antialias(widget);
 		FcPatternAddBool(pattern, FC_ANTIALIAS, antialias);
 	}
+	if (explicit_antialias != VTE_ANTI_ALIAS_USE_DEFAULT) {
+		FcBool aa;
+		if (FcPatternGetBool(pattern, FC_ANTIALIAS, 0,
+				     &aa) != FcResultNoMatch) {
+			FcPatternDel(pattern, FC_ANTIALIAS);
+		}
+		aa = (explicit_antialias == VTE_ANTI_ALIAS_FORCE_ENABLE) ?
+		     FcTrue : FcFalse;
+		FcPatternAddBool(pattern, FC_ANTIALIAS, aa);
+	}
 
 	/* Pick up the hinting setting. */
 	if (FcPatternGetBool(pattern, FC_HINTING, 0,
@@ -345,6 +403,7 @@ _vte_fc_defaults_from_rdb(GtkWidget *widget, FcPattern *pattern)
 gboolean
 _vte_fc_patterns_from_pango_font_desc(GtkWidget *widget,
 				      const PangoFontDescription *font_desc,
+				      VteTerminalAntiAlias antialias,
 				      GArray *pattern_array,
 				      _vte_fc_defaults_cb defaults_cb,
 				      gpointer defaults_data)
@@ -369,10 +428,13 @@ _vte_fc_patterns_from_pango_font_desc(GtkWidget *widget,
 	FcConfigSubstitute(NULL, pattern, FcMatchPattern);
 
 	/* Add any defaults configured for GTK+. */
-	_vte_fc_defaults_from_gtk(widget, pattern);
+	_vte_fc_defaults_from_gtk(widget, pattern, antialias);
 
 	/* Add defaults configured via the resource database. */
-	_vte_fc_defaults_from_rdb(widget, pattern);
+	_vte_fc_defaults_from_rdb(widget, pattern, antialias);
+
+	/* Add any hard-coded default for antialiasing. */
+	_vte_fc_set_antialias(pattern, antialias);
 
 	/* Add any defaults which are hard-coded in fontconfig. */
 	FcDefaultSubstitute(pattern);
@@ -390,7 +452,8 @@ _vte_fc_patterns_from_pango_font_desc(GtkWidget *widget,
 			tmp = FcFontRenderPrepare(NULL,
 						  pattern,
 						  fontset->fonts[i]);
-			_vte_fc_defaults_from_gtk(widget, tmp);
+			_vte_fc_defaults_from_gtk(widget, tmp, antialias);
+			_vte_fc_set_antialias(tmp, antialias);
 			save = FcPatternDuplicate(tmp);
 			FcPatternDestroy(tmp);
 			g_array_append_val(pattern_array, save);
@@ -404,7 +467,8 @@ _vte_fc_patterns_from_pango_font_desc(GtkWidget *widget,
 		match = FcFontMatch(NULL, pattern, &result);
 		if (result == FcResultMatch) {
 			tmp = FcPatternDuplicate(match);
-			_vte_fc_defaults_from_gtk(widget, tmp);
+			_vte_fc_defaults_from_gtk(widget, tmp, antialias);
+			_vte_fc_set_antialias(tmp, antialias);
 			save = FcPatternDuplicate(tmp);
 			FcPatternDestroy(tmp);
 			g_array_append_val(pattern_array, save);
