@@ -16,7 +16,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vteglyph.c,v 1.13 2003/05/28 21:53:59 nalin Exp $"
 
 #include "../config.h"
 
@@ -56,7 +55,7 @@ _vte_glyph_cache_new(void)
 	struct _vte_glyph_cache *ret;
 	int error;
 
-	ret = g_malloc(sizeof(struct _vte_glyph_cache));
+	ret = g_slice_new(struct _vte_glyph_cache);
 
 	ret->patterns = g_array_new(TRUE, TRUE, sizeof(FcPattern*));
 	ret->faces = NULL;
@@ -130,7 +129,7 @@ _vte_glyph_cache_free(struct _vte_glyph_cache *cache)
 	cache->width = 0;
 	cache->height = 0;
 	cache->ascent = 0;
-	g_free(cache);
+	g_slice_free(struct _vte_glyph_cache, cache);
 }
 
 void
@@ -138,6 +137,7 @@ _vte_glyph_cache_set_font_description(GtkWidget *widget,
 				      FcConfig *config,
 				      struct _vte_glyph_cache *cache,
 				      const PangoFontDescription *fontdesc,
+				      VteTerminalAntiAlias antialias,
 				      _vte_fc_defaults_cb defaults_cb,
 				      gpointer defaults_data)
 {
@@ -155,7 +155,9 @@ _vte_glyph_cache_set_font_description(GtkWidget *widget,
 
 	/* Convert the font description to a sorted set of patterns. */
 	patterns = g_array_new(TRUE, TRUE, sizeof(FcPattern*));
-	if (!_vte_fc_patterns_from_pango_font_desc(widget, fontdesc, patterns,
+	if (!_vte_fc_patterns_from_pango_font_desc(widget, fontdesc,
+						   antialias,
+						   patterns,
 						   defaults_cb,
 						   defaults_data)) {
 		g_array_free(patterns, TRUE);
